package com.usky.fire.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.fire.domain.PatrolInspectionSite;
import com.usky.fire.domain.SiteContent;
import com.usky.fire.service.PatrolInspectionContentService;
import com.usky.fire.service.PatrolInspectionSiteService;
import com.usky.fire.service.vo.PatrolInspectionAreaVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 巡查自检-巡检地点
 *
 * @author JCB
 * @since 2022-07-13
 */
@RestController
@RequestMapping("/patrolInspectionSite")
public class PatrolInspectionSiteController {

    @Autowired
    private PatrolInspectionSiteService patrolInspectionSiteService;
    @Autowired
    private PatrolInspectionContentService patrolInspectionContentService;


    /**
     * 巡查自检-巡检地点-左侧树形区域
     *
     * @return
     */
    @GetMapping("areaLeftList")
    public ApiResult<List<PatrolInspectionAreaVo>> areaLeftList() {
        return ApiResult.success(patrolInspectionSiteService.areaLeftList());
    }


    /**
     * 巡查自检-巡检地点-巡检列表
     *
     * @param areaId   区域id
     * @param siteName 地点名称
     * @param pageNum  当前页
     * @param pageSize 每页条数
     * @param id       巡检地点ID
     * @return
     */
    @GetMapping("patrolInspectionSiteList")
    public ApiResult<CommonPage<PatrolInspectionSite>> patrolInspectionSiteList(@RequestParam(value = "areaId", required = false) Integer areaId,
                                                                                @RequestParam(value = "siteName", required = false) String siteName,
                                                                                @RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                                                                @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize,
                                                                                @RequestParam(value = "id", required = false, defaultValue = "0") Integer id) {
        return ApiResult.success(patrolInspectionSiteService.patrolInspectionSiteList(areaId, siteName, pageNum, pageSize, id));
    }

    /**
     * 巡查自检-巡检地点-添加地点-保存
     *
     * @param PatrolInspectionSiteList
     * @return
     */
    @PostMapping("updatePatrolInspectionSiteList")
    public ApiResult<Void> updatePatrolInspectionSiteList(@RequestBody List<PatrolInspectionSite> PatrolInspectionSiteList) {
        patrolInspectionSiteService.updatePatrolInspectionSiteList(PatrolInspectionSiteList);
        return ApiResult.success();
    }

    /**
     * 巡查自检-巡检地点-编辑
     *
     * @param PatrolInspectionSite
     * @return
     */
    @PostMapping("updatePatrolInspectionSite")
    public ApiResult<Void> updatePatrolInspectionSite(@RequestBody PatrolInspectionSite PatrolInspectionSite) {
        patrolInspectionSiteService.updatePatrolInspectionSite(PatrolInspectionSite);
        return ApiResult.success();
    }

    /**
     * 巡查自检-巡检地点-删除
     *
     * @param id 巡检地点ID
     * @return
     */
    @GetMapping("delPatrolInspectionSite")
    public ApiResult<Void> delPatrolInspectionSite(@RequestParam(value = "id") Integer id) {
        patrolInspectionSiteService.delPatrolInspectionSite(id);
        return ApiResult.success();
    }


    /**
     * 巡查自检-巡检地点-分配内容-列表查询
     *
     * @param pageNum  当前页
     * @param pageSize 每页条数
     * @param siteId   巡检地点ID
     * @return
     */
    @GetMapping("siteContentList")
    public ApiResult<Map<String, Object>> siteContentList(@RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                                          @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize,
                                                          @RequestParam(value = "siteId") Integer siteId) {
        Map<String, Object> ajax = new HashMap<>();
        ajax.put("checkedKeys", patrolInspectionSiteService.siteContentList(siteId));
        ajax.put("InspectionContent", patrolInspectionContentService.patrolInspectionContentList(null, pageNum, pageSize, 0));
        return ApiResult.success(ajax);
    }

    /**
     * 巡查自检-巡检地点-分配内容-保存
     *
     * @param siteContentList
     * @return
     */
    @PostMapping("addSiteContent")
    public ApiResult<Void> addSiteContent(@RequestBody List<SiteContent> siteContentList) {
        patrolInspectionSiteService.addSiteContent(siteContentList);
        return ApiResult.success();
    }

}

